package de.cliffbreak.varo.listeners;

import java.util.Stack;

import com.destroystokyo.paper.event.server.PaperServerListPingEvent;

import org.bukkit.Bukkit;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.server.ServerListPingEvent;
import org.bukkit.scheduler.BukkitRunnable;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.uitls.MessageUtils;
import de.cliffbreak.varo.uitls.PacketReader;

public class PlayerJoinQuitListener implements Listener {

    private final Varo plugin;

    public PlayerJoinQuitListener(final Varo plugin) {
        this.plugin = plugin;
    }

    @EventHandler
    public void onPlayerQuit(final PlayerQuitEvent e) {
        e.setQuitMessage(null);
        if (!this.plugin.getBanUtils().isBanned(e.getPlayer().getUniqueId())) {
            this.plugin.getNPCManager().createClone(e.getPlayer(), true);
            Bukkit.broadcast(
                    MessageUtils.getRichTextComponent(e.getPlayer().getName(), "§f hat den Server verlassen.", true));
        }
    }

    @EventHandler
    public void onPlayerJoin(final PlayerJoinEvent e) {
        e.setJoinMessage(null);
        this.plugin.getNPCManager().removeClone(e.getPlayer());
        this.plugin.getNPCManager().syncClones(e.getPlayer());

        // Inject PacketReader
        final PacketReader packetReader = new PacketReader(this.plugin, e.getPlayer());
        packetReader.inject();

        Bukkit.broadcast(
                MessageUtils.getRichTextComponent(e.getPlayer().getName(), "§f hat den Server betreten.", true));

        if (this.plugin.getConfig().getBoolean("Varo.Debug")) {

            e.getPlayer().sendMessage("\n§7§l#### §9Cliffbreak.de - §lVaro §r§9Changelog §7§l####\n \n"
                    + "   §cWarning: §r§c Plugin is running in DEBUG mode!\n ");

            if (!(e.getPlayer().getName().equals("AragurLP") || e.getPlayer().getName().equals("AragurTV"))) {

                final Stack<String> changes = new Stack<String>();

                changes.push(" §7• §r§lADD: §rAdd /varo portal to allow Operators to create a Portal");
                changes.push(" §7• §r§lADD: §rPlayers can't create Portals");
                changes.push(" §7• §r§lADD: §rFakePlayer now drops his items and gets real player banned on death");
                changes.push(" §7• §r§lADD: §rFakePlayer is no attackable (can also be killed)");
                changes.push(" §7• §r§lADD: §rAdd a FakePlayer if Player is logged out");
                changes.push(" §7• §r§lADD: §rAdd /varo test && /varo testremove");
                changes.push(" §7• §r§lFIX: §r/varo Command is only usable by Server Operators now");
                changes.push(
                        " §7• §r§lADD: §rDisallow special Items (Enchanted Golden Apple, Fishing Rod, Totem of Undying)");
                changes.push(" §7• §r§lADD: §rUse Vanilla Hearts");
                changes.push(" §7• §r§lADD: §rDisable Fishing Rod Crafting Recipe");
                changes.push(" §7• §r§lFIX: §rCheck for Ban in AsyncPreLoginEvent");
                changes.push(" §7• §r§lFIX: §rOnly kick player after Entity isn't ticking anymore");
                changes.push(" §7• §r§lADD: §r/varo Command for Admins");
                changes.push(" §7• §r§lADD: §rBan Player on Death");
                changes.push(" §7• §r§lADD: §r1.8 Health Regeneration");
                changes.push(" §7• §r§lADD: §rCustom Dynamic ServerListEntry");
                changes.push(" §7• §r§lADD: §rControl Difficulty and Gamerules by the Plugin");
                changes.push(" §7• §r§lADD: §rHover over Playername for Details (Team)");
                changes.push(" §7• §r§lADD: §rRedesigned Chat with Timestamp");

                new BukkitRunnable() {
                    @Override
                    public void run() {
                        if (!e.getPlayer().isOnline()) {
                            this.cancel();
                            return;
                        }
                        if (changes.empty()) {
                            this.cancel();
                            return;
                        }
                        e.getPlayer().sendMessage(changes.pop());
                    }
                }.runTaskTimer(this.plugin, 20, 20);
            }
        }

        new BukkitRunnable() {
            @Override
            public void run() {
                e.getPlayer().setResourcePack("https://cdn.cliffbreak.de/vanilla.zip",
                        "cf29612c4506d02fd1e83ae61d3a660a1f4b9bcd");
            }
        }.runTaskLater(plugin, 5); // wait 5 ticks
    }

    @EventHandler
    public void onServerListPing(final ServerListPingEvent event) {
        if (event instanceof PaperServerListPingEvent) {
            handlePaperServerListPing((PaperServerListPingEvent) event);
        } else {
            this.plugin.getLogger().info("Error: Plugin is not running on a PaperServer");
        }
    }

    private void handlePaperServerListPing(final PaperServerListPingEvent e) {
        e.setHidePlayers(true);
        // TODO: Add day counter
        e.setMotd("§9§l" + "VARO" + "§b§l " + "Tag: -1" + "\n§r§7" + "powered by " + "§4" + "Cliffbreak.de");
    }
}